searchNodes=[{"doc":"A Broadway Producer for Polyn. The word Producer here is confusing because the word is overloaded. In this module Producer refers to GenStage data pipelines where a :producer is the stage that receives demand for data and sends it to a :consumer . This module doesn't &quot;produce&quot; new events that get added to the NATS server for other services to consume. Rather it consumes existing events from a NATS Stream and passes them to GenStage :consumer modules in one application. Usage This module wraps OffBroadway.Jetstream.Producer and will validate that any messages coming through are valid events and conform to the schema for the event. Use the OffBroadway.Jetstream.Producer documentation to learn how to use it. The only difference being you will use OffBroadway.Polyn.Producer in your :module configuration instead of the Jetsteram one. Invalid messages will send an ACKTERM to the NATS server so that they aren't sent again. They will be marked as failed and removed from the pipeline. Valid messages that come in a batch with an invalid message will send a NACK response before an error is raised so that the NATS server will know they were received but need to be sent again Example defmodule MyBroadway do use Broadway def start_link ( _opts ) do Broadway . start_link ( __MODULE__ , name : MyBroadway , producer : [ module : { OffBroadway.Polyn.Producer , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; } , concurrency : 10 ] , processors : [ default : [ concurrency : 10 ] ] , batchers : [ example : [ concurrency : 5 , batch_size : 10 , batch_timeout : 2_000 ] ] ) end def handle_message ( _processor_name , message , _context ) do message |&gt; Message . update_data ( &amp; process_data / 1 ) |&gt; Message . put_batcher ( :example ) end end","ref":"OffBroadway.Polyn.Producer.html","title":"OffBroadway.Polyn.Producer","type":"module"},{"doc":"Polyn is a dead simple service framework designed to be language agnostic while providing a simple, yet powerful, abstraction layer for building reactive events based services.","ref":"Polyn.html","title":"Polyn","type":"module"},{"doc":"Publish an event to the message bus. Will validate the data against an existing schema added by Polyn CLI. Options :source - The source of the event. By default will be the domain combined with the source_root :triggered_by - The event that triggered this one. Will use information from the event to build up the polyntrace data See Gnat.pub/4 for other options Examples iex&gt; Polyn . pub ( :gnat , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } ) :ok iex&gt; Polyn . pub ( :gnat , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } , source : &quot;admin&quot; ) :ok","ref":"Polyn.html#pub/4","title":"Polyn.pub/4","type":"function"},{"doc":"Reply to an event you've subscribed to that included a reply_to option. Options :source - The source of the event. By default will be the domain combined with the source_root :triggered_by - The event that triggered this one. Will use information from the event to build up the polyntrace data See Gnat.pub/4 for other options Examples iex&gt; Polyn . reply ( :gnat , &quot;INBOX.me&quot; , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } ) :ok iex&gt; Polyn . reply ( :gnat , &quot;INBOX.me&quot; , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } , source : &quot;admin&quot; ) :ok","ref":"Polyn.html#reply/5","title":"Polyn.reply/5","type":"function"},{"doc":"Issue a request in a psuedo-synchronous fashion. Requests still require an event be defined in the schema store. The event you send and receive will both be validated Options :source - The source of the event. By default will be the domain combined with the source_root :triggered_by - The event that triggered this one. Will use information from the event to build up the polyntrace data See Gnat.request/4 for other options Examples iex&gt; Polyn . request ( :gnat , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } ) { :ok , %{ body : % Event { } } } iex&gt; Polyn . request ( :gnat , &quot;user.created.v1&quot; , %{ name : &quot;Mary&quot; } , source : &quot;admin&quot; ) { :ok , %{ body : % Event { } } }","ref":"Polyn.html#request/4","title":"Polyn.request/4","type":"function"},{"doc":"Options you can pass to most Polyn module functions :source - The source of the event. By default will be the domain combined with the source_root :triggered_by - The event that triggered this one. Will use information from the event to build up the polyntrace data","ref":"Polyn.html#t:polyn_options/0","title":"Polyn.polyn_options/0","type":"type"},{"doc":"Options for publishing events. See Gnat.pub/4 for more info :headers - Headers to include in the message :reply_to - Subject to send a response to","ref":"Polyn.html#t:pub_options/0","title":"Polyn.pub_options/0","type":"type"},{"doc":"Options for publishing events. See Gnat.request/4 for more info :headers - Headers to include in the message :receive_timeout - How long to wait for a response","ref":"Polyn.html#t:req_options/0","title":"Polyn.req_options/0","type":"type"},{"doc":"The Event structure used throughout Polyn.","ref":"Polyn.Event.html","title":"Polyn.Event","type":"module"},{"doc":"","ref":"Polyn.Event.html#full_source/0","title":"Polyn.Event.full_source/0","type":"function"},{"doc":"Get the Event source prefixed with reverse domain name","ref":"Polyn.Event.html#full_source/1","title":"Polyn.Event.full_source/1","type":"function"},{"doc":"Get the Event type prefixed with reverse domain name","ref":"Polyn.Event.html#full_type/1","title":"Polyn.Event.full_type/1","type":"function"},{"doc":"Create a new Polyn.Event","ref":"Polyn.Event.html#new/1","title":"Polyn.Event.new/1","type":"function"},{"doc":"Generate a new event id","ref":"Polyn.Event.html#new_event_id/0","title":"Polyn.Event.new_event_id/0","type":"function"},{"doc":"Generate a new timestamp for the event","ref":"Polyn.Event.html#new_timestamp/0","title":"Polyn.Event.new_timestamp/0","type":"function"},{"doc":"Previous events that led to this one","ref":"Polyn.Event.html#t:polyntrace/0","title":"Polyn.Event.polyntrace/0","type":"type"},{"doc":"The Event structure used throughout Polyn. id - Identifies the event. specversion - The version of the CloudEvents specification which the event uses. type - Describes the type of event related to the originating occurrence. data - The event payload. dataschema - Identifies the schema that data adheres to. datacontenttype - Content type of the data value. Must adhere to RFC 2046 format. source - Identifies the context in which an event happened. time - Timestamp of when the occurrence happened. Must adhere to RFC 3339. polyntrace - Previous events that led to this one polyndata - Information about the client that produced the event and additional data","ref":"Polyn.Event.html#t:t/0","title":"Polyn.Event.t/0","type":"type"},{"doc":"Utilities for working with Polyn naming conventions","ref":"Polyn.Naming.html","title":"Polyn.Naming","type":"module"},{"doc":"Convert a colon separated name into a dot separated name Examples iex&gt; Polyn.Naming . colon_to_dot ( &quot;com:acme:user:created:v1:schema:v1&quot; ) &quot;com.acme.user.created.v1.schema.v1&quot;","ref":"Polyn.Naming.html#colon_to_dot/1","title":"Polyn.Naming.colon_to_dot/1","type":"function"},{"doc":"Convert a dot separated name into a colon separated name Examples iex&gt; Polyn.Naming . dot_to_colon ( &quot;com.acme.user.created.v1.schema.v1&quot; ) &quot;com:acme:user:created:v1:schema:v1&quot;","ref":"Polyn.Naming.html#dot_to_colon/1","title":"Polyn.Naming.dot_to_colon/1","type":"function"},{"doc":"Remove the :domain prefix from a name Examples iex&gt; Polyn.Naming . trim_domain_prefix ( &quot;com:acme:user:created:v1:schema:v1&quot; ) &quot;user:created:v1:schema:v1&quot; iex&gt; Polyn.Naming . trim_domain_prefix ( &quot;com.acme.user.created.v1.schema.v1&quot; ) &quot;user.created.v1.schema.v1&quot;","ref":"Polyn.Naming.html#trim_domain_prefix/1","title":"Polyn.Naming.trim_domain_prefix/1","type":"function"},{"doc":"Remove the version suffix from a name Examples iex&gt; Polyn.Naming . trim_version_suffix ( &quot;com.acme.user.created.v1&quot; ) &quot;com.acme.user.created&quot; iex&gt; Polyn.Naming . trim_version_suffix ( &quot;com:acme:user:created:v1&quot; ) &quot;com:acme:user:created&quot;","ref":"Polyn.Naming.html#trim_version_suffix/1","title":"Polyn.Naming.trim_version_suffix/1","type":"function"},{"doc":"Validate the name of an event, also sometimes called an event type . Examples iex&gt; Polyn.Naming . validate_event_name ( &quot;user.created&quot; ) :ok iex&gt; Polyn.Naming . validate_event_name ( &quot;user created&quot; ) { :error , [ &quot;Event names can&#39;t have spaces&quot; ] }","ref":"Polyn.Naming.html#validate_event_name/1","title":"Polyn.Naming.validate_event_name/1","type":"function"},{"doc":"Validate the source of an event Examples iex&gt; Polyn.Naming . validate_source_name ( &quot;user.created&quot; ) :ok iex&gt; Polyn.Naming . validate_source_name ( &quot;user:created&quot; ) :ok iex&gt; Polyn.Naming . validate_source_name ( &quot;user created&quot; ) { :error , [ &quot;Source names can&#39;t have spaces&quot; ] }","ref":"Polyn.Naming.html#validate_source_name/1","title":"Polyn.Naming.validate_source_name/1","type":"function"},{"doc":"Give a version number to a name Examples iex&gt; Polyn.Naming . version_suffix ( &quot;com:acme:user:created:&quot; ) &quot;com:acme:user:created:v1&quot; iex&gt; Polyn.Naming . version_suffix ( &quot;com.acme.user.created.&quot; , 2 ) &quot;com.acme.user.created.v2&quot;","ref":"Polyn.Naming.html#version_suffix/2","title":"Polyn.Naming.version_suffix/2","type":"function"},{"doc":"Use Polyn.PullConsumer to connect and process messages from an existing NATS Consumer that was setup with Polyn CLI . This module is a wrapper around Jetstream.PullConsumer that does schema validation with the received messages. This type of Consumer is meant for simple use cases that don't involve concurrency or batching.","ref":"Polyn.PullConsumer.html","title":"Polyn.PullConsumer","type":"behaviour"},{"doc":"Returns a specification to start this module under a supervisor. See the &quot;Child specification&quot; section in the Supervisor module for more detailed information.","ref":"Polyn.PullConsumer.html#child_spec/1","title":"Polyn.PullConsumer.child_spec/1","type":"function"},{"doc":"Closes the pull consumer and stops underlying process. Example { :ok , consumer } = PullConsumer . start_link ( ExamplePullConsumer , connection_name : :gnat , stream_name : &quot;TEST_STREAM&quot; , consumer_name : &quot;TEST_CONSUMER&quot; ) :ok = PullConsumer . close ( consumer )","ref":"Polyn.PullConsumer.html#close/1","title":"Polyn.PullConsumer.close/1","type":"function"},{"doc":"Invoked to synchronously process a message pulled by the consumer. Depending on the value it returns, the acknowledgement is or is not sent. Polyn will deserialize the message body into a Polyn.Event struct and use that as the first argument, followed by the original message, follwed by the state. ACK actions See Jetstream.PullConsumer.handle_message/2 for available options Example def handle_message ( event , _message , state ) do IO . inspect ( event ) { :ack , state } end","ref":"Polyn.PullConsumer.html#c:handle_message/3","title":"Polyn.PullConsumer.handle_message/3","type":"callback"},{"doc":"Invoked when the server is started. start_link/3 or start/3 will block until it returns. init_arg is the argument term (second argument) passed to start_link/3 . See Connection.init/1 for more details.","ref":"Polyn.PullConsumer.html#c:init/1","title":"Polyn.PullConsumer.init/1","type":"callback"},{"doc":"Starts a Jetstream.PullConsumer process without links (outside of a supervision tree). See start_link/3 for more information.","ref":"Polyn.PullConsumer.html#start/3","title":"Polyn.PullConsumer.start/3","type":"function"},{"doc":"Starts a pull consumer linked to the current process with the given function. This is often used to start the pull consumer as part of a supervision tree. Once the server is started, the init/1 function of the given module is called with init_arg as its argument to initialize the server. To ensure a synchronized start-up procedure, this function does not return until init/1 has returned. See GenServer.start_link/3 for more details.","ref":"Polyn.PullConsumer.html#start_link/3","title":"Polyn.PullConsumer.start_link/3","type":"function"},{"doc":"A GenServer wrapper to use when working with vanilla NATS subscriptions outside of JetStream. This process will hang around and listen for messages to come in and then trigger a handle_message/3 callback defmodule MySubscriber do use Polyn.Subscriber def start_link ( init_args ) do Polyn.Subscriber . start_link ( __MODULE__ , init_args , connection_name : :gnat , event : &quot;user.created.v1&quot; ) end def init ( _arg ) do { :ok , nil } end def handle_message ( event , message , state ) do # Do something cool with the event { :noreply , state } end end","ref":"Polyn.Subscriber.html","title":"Polyn.Subscriber","type":"behaviour"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Polyn.Subscriber.html#child_spec/1","title":"Polyn.Subscriber.child_spec/1","type":"function"},{"doc":"Called when the subscribed event is published. Return the same values as you would for a Genserver.handle_info/2 callback","ref":"Polyn.Subscriber.html#c:handle_message/3","title":"Polyn.Subscriber.handle_message/3","type":"callback"},{"doc":"Options :connection_name - Required, The pid or name of Gnat connection :event - Required, The name of the event to subscribe to :queue_group - a string that identifies which queue group you want to join Any other passed options will be assumed to be GenServer options","ref":"Polyn.Subscriber.html#start_link/3","title":"Polyn.Subscriber.start_link/3","type":"function"},{"doc":"","ref":"Polyn.Subscriber.html#t:start_options/0","title":"Polyn.Subscriber.start_options/0","type":"type"},{"doc":"Error raised when schemas are not found where they are expected to be","ref":"Polyn.SchemaException.html","title":"Polyn.SchemaException","type":"exception"},{"doc":"Error raised when Event data is invalid","ref":"Polyn.ValidationException.html","title":"Polyn.ValidationException","type":"exception"},{"doc":"Polyn is a dead simple service framework designed to be language agnostic while providing a simple, yet powerful, abstraction layer for building reactive events based services.","ref":"readme.html","title":"Polyn","type":"extras"},{"doc":"According to Jonas Boner , reactive Microservices require you to: Follow the principle “do one thing, and one thing well” in defining service boundaries Isolate the services Ensure services act autonomously Embrace asynchronous message passing Stay mobile, but addressable Design for the required level of consistency Polyn implements this pattern in a manner that can be applied to multiple programming languages, such as Ruby, Elixir, or Python, enabling you to build services that can communicate regardless of the language you use. Using an event-based microservice architecture is a great way to decouple your services, create reliability, and scalability. However, there is no standard way to format events which creates entropy and inconsistency between services, requiring developers to create different event handling logic for each event type they consume. Polyn solves this problem by creating and enforcing a consistent event format on both the producer and consumer-side so all the services in your system can focus their effort on the data rather than the event format. Rather than defining its own event schema, Polyn uses the Cloud Events specification and strictly enforces the event format. This means that you can use Polyn to build services that can be used by other services, or natively interact with things such as GCP Cloud Functions. For events that include data Polyn also leverages the JSON Schema specification to create consistency.","ref":"readme.html#philosophy","title":"Polyn - Philosophy","type":"extras"},{"doc":"In order for Polyn to process and validate event schemas you will need to use Polyn CLI to create an events codebase. Once your events codebase is created you can create and manage your schemas there.","ref":"readme.html#schema-creation","title":"Polyn - Schema Creation","type":"extras"},{"doc":"Domain The Cloud Event Spec specifies that every event &quot;SHOULD be prefixed with a reverse-DNS name.&quot; This name should be consistent throughout your organization. You define that domain like this: config :polyn , :domain , &quot;app.spiff&quot; Event Source Root The Cloud Event Spec specifies that every event MUST have a source attribute and recommends it be an absolute URI . Your application must configure the source_root to use for events produced at the application level. Each event producer can include its own source to append to the source_root if it makes sense. config :polyn , :source_root , &quot;orders.payments&quot;","ref":"readme.html#configuration","title":"Polyn - Configuration","type":"extras"},{"doc":"Publishing Messages Use Polyn.pub/4 to publish new events to the server Simple Stream Consumption If you have use case that doesn't require batching or concurrency you can use Polyn.PullConsumer to receive messages one at a time Complex Stream Consumption If you have a complex use case requiring batching or concurrency you should use the OffBroadway.Polyn.Producer to create a data pipeline for your messages. Vanilla NATS Subscription If there are events you want to subscribe to that are more ephemeral or don't need JetStream functionality you can use the Polyn.Subscriber module to setup a process to subscribe and handle those events Request-Reply You can use Polyn.request/4 to a do a psuedo-synchronous request . You can subscribe to an event using a Polyn.Subscriber and reply using Polyn.reply/5 . Both your request and your reply will need schema definitions and will be validated against them.","ref":"readme.html#usage","title":"Polyn - Usage","type":"extras"},{"doc":"If available in Hex , the package can be installed by adding polyn to your list of dependencies in mix.exs : def deps do [ { :polyn , &quot;~&gt; 0.1.0&quot; } ] end Optional Broadway Dependency To use the OffBroadway.Polyn.Producer you'll also need to add a Broadway as a dependency","ref":"readme.html#installation","title":"Polyn - Installation","type":"extras"},{"doc":"Documentation can be generated with ExDoc and published on HexDocs . Once published, the docs can be found at https://hexdocs.pm/polyn .","ref":"readme.html#documentation","title":"Polyn - Documentation","type":"extras"}]